#pragma once
#include "my_entity_Bump.h"
#include "tangentSpace.h"

#define DEGTORAD(degree) ((degree) * (3.141592654f / 180.0f))

namespace SceneManagement
{

float lightPosition0[] = { .0f, .0f, 2.0f, 1.0f };
float diffuse_light0[]  = { 0.5f, 0.1f, 0.5f, 1.0f };

Vertex surface[NUM_VERTICES] =
{
//    tu   tv     r    g    b    a		nx    ny    nz       x     y     z 
    { 0.0f,0.0f,  1.0f,1.0f,1.0f,1.0f,  0.0f, 0.0f, 1.0f,  -1.0f,-1.0f, 0.0f },
    { 1.0f,0.0f,  1.0f,1.0f,1.0f,1.0f,	0.0f, 0.0f, 1.0f,   1.0f,-1.0f, 0.0f },
    { 1.0f,1.0f,  1.0f,1.0f,1.0f,1.0f,	0.0f, 0.0f, 1.0f,   1.0f, 1.0f, 0.0f },
    { 0.0f,1.0f,  1.0f,1.0f,1.0f,1.0f,	0.0f, 0.0f, 1.0f,  -1.0f, 1.0f, 0.0f }

	//rgba vill be used to compute Vertex->Light vector
};

Vector3f vTangent[4];
Vector3f vBiNormal[4];
Vector3f vNormal[4];
Matrix4f invTangentMatrix;

My_Entity_Bump::My_Entity_Bump(unsigned int tDiffuse, unsigned int tNormal)
{
	glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
	tex1 = tDiffuse;
	tex2 = tNormal;
	for (int i=0; i<NUM_VERTICES; i++)
	{
		TangentSpace::createTangentSpaceVectors(&Vector3f(surface[(0+i)%NUM_VERTICES].x, surface[(0+i)%NUM_VERTICES].y, surface[(0+i)%NUM_VERTICES].z),   //current vertex
												&Vector3f(surface[(-1+i)%NUM_VERTICES].x, surface[(-1+i)%NUM_VERTICES].y, surface[(-1+i)%NUM_VERTICES].z), //previous vertex
												&Vector3f(surface[(1+i)%NUM_VERTICES].x, surface[(1+i)%NUM_VERTICES].y, surface[(1+i)%NUM_VERTICES].z),   //next vertex
												surface[(0+i)%NUM_VERTICES].tu, surface[(0+i)%NUM_VERTICES].tv,     //current tex-coords
												surface[(-1+i)%NUM_VERTICES].tu, surface[(-1+i)%NUM_VERTICES].tv,   //prev tex-coords
												surface[(1+i)%NUM_VERTICES].tu, surface[(1+i)%NUM_VERTICES].tv,     //next tex-coords
												&vTangent[i], &vBiNormal[i], &vNormal[i]);
	}
	//
	// Build up an inverse tangent-space matrix using the Tangent, 
	// Binormal, and Normal calculated for the current vertex and 
	// then use it to transform our L vector (of N.L), which is in 
	// object-space, into tangent-space.
	//
	// A tangent matrix is of the form:
	//
	// |Tx Bx Nx 0|
	// |Ty By Ny 0|
	// |Tz Bz Nz 0|
	// |0  0  0  1|
	//
	// Note: Our vectors have already been inverted, so there is no 
	// need to invert our tangent matrix once we build it up.
	//
}

My_Entity_Bump::~My_Entity_Bump(void)
{
	if (tex1>0)
		glDeleteTextures(1, &tex1);
	if (tex2>0)
		glDeleteTextures(1, &tex2);
}


Vector3f scaleAndBiasVectorAsRGBColor(Vector3f* vVector )
{
    Vector3f vScaledAndBiasedVector;

    vScaledAndBiasedVector.x = ((*vVector).x * 0.5f) + 0.5f;
    vScaledAndBiasedVector.y = ((*vVector).y * 0.5f) + 0.5f;
    vScaledAndBiasedVector.z = ((*vVector).z * 0.5f) + 0.5f;

    return vScaledAndBiasedVector;
}

void My_Entity_Bump::Draw()
{

        //
        // Spin our point light around the cube...
        //

        static float fAngle = 0;
        fAngle += .125f;
        // Wrap it around, if it gets too big
        while(fAngle > 360.0f) fAngle -= 360.0f;
        while(fAngle < 0.0f)   fAngle += 360.0f;
        float x = sinf( DEGTORAD(fAngle) );
        float y = cosf( DEGTORAD(fAngle) );

        // The call to glLightfv is just like calling glTranslatef on our
        // light's position.
        lightPosition0[0] = 1.2f * x;
        lightPosition0[1] = 1.2f * y;
        lightPosition0[2] = 2.0f;
/*        lightPosition0[0] = 1.0f * x;
        lightPosition0[1] = 0.0f;
        lightPosition0[2] = 15.0f;
*/



	glPushAttrib(GL_COLOR_MATERIAL | GL_LIGHTING);	
	
	// For lighting, pull our material colors from the vertex color...
    glColorMaterial( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
    glEnable( GL_COLOR_MATERIAL );

	// Set light 0 to be a pure white point light
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0 );
    glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuse_light0 );
    glLightfv( GL_LIGHT0, GL_POSITION, lightPosition0 );


    GLfloat ambient_lightModel[] = { 0.3f, 0.3f, 0.3f, 0.2f };
    glLightModelfv( GL_LIGHT_MODEL_AMBIENT, ambient_lightModel );
	//glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
	//glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,GL_FALSE);

	glActiveTexture(GL_TEXTURE0);
    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glDisable(GL_TEXTURE_3D_EXT);
    glDisable(GL_TEXTURE_CUBE_MAP_ARB);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_TEXTURE_GEN_R);

	glActiveTexture(GL_TEXTURE1);
    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glDisable(GL_TEXTURE_3D_EXT);
    glDisable(GL_TEXTURE_CUBE_MAP_ARB);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_TEXTURE_GEN_R);

	glActiveTexture(GL_TEXTURE0);
    glBindTexture (GL_TEXTURE_2D, tex2);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);      
    glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_DOT3_RGB_EXT);		// Perform a Dot3 operation...
    glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);           // between the N (of N.L) which is stored in a normal map texture...
    glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
    glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_PRIMARY_COLOR_EXT); // with the L (of N.L) which is stored in the vertex's diffuse color.
    glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND1_RGB_EXT, GL_SRC_COLOR);

    glActiveTexture(GL_TEXTURE1);
    glBindTexture (GL_TEXTURE_2D, tex1);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);  // Modulate...
    glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_MODULATE);
    glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_PREVIOUS_EXT);
    glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);    // the color argument passed down from the previous stage (stage 0) with...
    glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_TEXTURE);
    glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND1_RGB_EXT, GL_SRC_COLOR);


    Vector3f vLightPosES;    // Light position (in eye-space)
    Vector3f vLightPosOS;    // Light position (in object-space)
    Vector3f vVertToLightOS; // L vector of N.L (in object-space)
    Vector3f vVertToLightTS; // L vector of N.L (in tangent-space)

	/*  -----   We have to get the light position from OpenGL

    // Transform the light's position from eye-space to object-space
    // Get the light's current position, which is in eye-space, if we get it from OpenGL 
	glGetLight(GL_LIGHT0, GL_POSITION, &vLightPosES[0]);

    // Transform the light's position from eye-space into object-space
    Matrix4f modelViewMatrix;
    Matrix4f modelViewMatrixInverse;
    glGetFloatv(GL_MODELVIEW_MATRIX, &modelViewMatrix.data[0] );
    modelViewMatrixInverse = modelViewMatrix.invertMatrix();
    vLightPosOS = vLightPosES;
    modelViewMatrixInverse.transformPoint( &vLightPosOS );

	*/

	//We have the light position
	vLightPosOS.x = lightPosition0[0];
    vLightPosOS.y = lightPosition0[1];
    vLightPosOS.z = lightPosition0[2];

	glBegin(GL_QUADS);

		Vector3f vCurrentVertex;
		for( int i = 0; i < NUM_VERTICES; ++i )
            {
                vCurrentVertex.x = surface[i].x;
                vCurrentVertex.y = surface[i].y;
                vCurrentVertex.z = surface[i].z;

                glMultiTexCoord2f( GL_TEXTURE0, surface[i].tu, surface[i].tv );
                glMultiTexCoord2f( GL_TEXTURE1, surface[i].tu, surface[i].tv );

                // For each vertex, rotate L (of N.L) into tangent-space and 
                // pass it into OpenGL's texture blending system by packing it 
                // into the glVertex3f for that vertex.

                vVertToLightOS = vLightPosOS - vCurrentVertex;
                vVertToLightOS.normalize();

				//Inverse Tangent matrix
				invTangentMatrix.data[0] = vTangent[i].x;
				invTangentMatrix.data[1] = vTangent[i].y;
				invTangentMatrix.data[2] = vTangent[i].z;

				invTangentMatrix.data[4] = vBiNormal[i].x;
				invTangentMatrix.data[5] = vBiNormal[i].y;
				invTangentMatrix.data[6] = vBiNormal[i].z;

				invTangentMatrix.data[8] = vNormal[i].x;
				invTangentMatrix.data[9] = vNormal[i].y;
				invTangentMatrix.data[10] = vNormal[i].z;

				invTangentMatrix.data[15] = 1;

                vVertToLightTS = vVertToLightOS;
                invTangentMatrix.transformVector( &vVertToLightTS );

                // Last but not least, we must scale and bias our L vector 
                // before passing it as a vertex color since colors have a 
                // clamped range of [0,1]. If we don't do this any negative 
                // components of our L vector will get clamped and the vector 
                // will be wrong. Of course, the hardware assumes that we are 
                // going to do this, so it will simply decode the original 
                // vector back out by reversing the scale and bias we've 
                // performed here.

                Vector3f vVertToLightTS_scaledAndBiased = scaleAndBiasVectorAsRGBColor( &vVertToLightTS );

                glColor3f( vVertToLightTS_scaledAndBiased.x, vVertToLightTS_scaledAndBiased.y, vVertToLightTS_scaledAndBiased.z );

                glNormal3f( surface[i].nx, surface[i].ny, surface[i].nz );
                glVertex3f( vCurrentVertex.x, vCurrentVertex.y, vCurrentVertex.z );
            }

		glEnd();

	//for (int i = 0; i < 2; i++)
	//{
	//	glActiveTexture(GL_TEXTURE0 + i);
	//	glLoadIdentity();
	//	glBindTexture(GL_TEXTURE_2D, 0);
	//	glDisable(GL_TEXTURE_2D);
	//}

	glActiveTexture(GL_TEXTURE0);


	glPopAttrib();
}

}
