#include "stdafx.h"
#include "BMPLoader.h"

namespace SceneManagement
{
	BMPLoader::BMPLoader(void)
	{
		width = 0;
		height = 0;
		bytes = NULL;
	}

	BMPLoader::~BMPLoader(void){delete[] bytes;}

	BYTE& BMPLoader::Pixel(int x,int y,int c){return bytes[(y*width+x)*3+c];}

	void BMPLoader::AllocateMem(){delete[] bytes;bytes=new BYTE[width*height*3];}

	std::string BMPLoader::TranslateBMPError(BMPError err)
	{
		switch(err)
		{
		case(BMPNOTABITMAP):
			return "This file is not a bitmap, specifically it doesn't start 'BM'";
		case(BMPNOOPEN):
			return "Failed to open the file, suspect it doesn't exist";
		case(BMPFILEERROR):
			return "ferror said we had an error. This error seems to not always mean anything, try ignoring it";
		case(BMPBADINT):
			return "sizeof(int)!=4 quite a lot of rewriting probably needs to be done on the code";
		case(BMPNOERROR):
			return "No errors detected";
		case(BMPUNKNOWNFORMAT):
			return "Unknown bmp format, ie not 24bit, 256,16 or 2 colour";
		default:
			return "Not a valid error code";
		}
	}

	BMPError BMPLoader::BMPLoad(std::string fname)
	{
		if(sizeof(int)!=4) return BMPBADINT;
			
		FILE* f=fopen(fname.c_str(),"rb");			//open for reading in binary mode
		if(!f) return BMPNOOPEN;
		char header[54];
		fread(header,54,1,f);						//read the 54bit main header

		if (!(header[0]=='B' && header[1]=='M'))
		{
			fclose(f);
			return BMPNOTABITMAP;					//all bitmaps should start "BM"
		}
		
		int offset=*(unsigned int*)(header+10);		//it seems gimp sometimes makes its headers small, so we have to do this. hence all the fseeks
		
		width=*(int*)(header+18);
		height=*(int*)(header+22);
		int shift = 0;
		if (((width * 3) % 4) == 3)
				shift = 1;
			else if (((width * 3) % 4) == 2)
				shift = 2;
			else if (((width * 3) % 4) == 1)
				shift = 3;
		BYTE* tmpBytes = new BYTE[2]; 
		AllocateMem();								//now the bitmap knows how big it is it can allocate its memory

		int bits=int(header[28]);					//colourdepth

		int x,y,c;
		BYTE cols[256*4];							//colourtable
		switch(bits)
		{
		case(24):
			fseek(f,offset,SEEK_SET);
			
			for (int y=0; y<height; y++)
			{
				fread(bytes+(y*width*3),width*3,1,f);		//nacte jeden radek bitmapy
				fread(tmpBytes, shift ,1,f);		//nacte zbyle byty kvuli zarovnani. Pro jistotu je ukladam.
			}

			//fread(bytes,width*height*3,1,f);		//24bit is easy
			//for(x=0;x<width*height*3;x+=3)			//except the format is BGR, grr
			//{
			//	BYTE temp=bytes[x];
			//	bytes[x]=bytes[x+2];
			//	bytes[x+2]=temp;
			//}
			break;

		case(8):
			fread(cols,256*4,1,f);							//read colortable
			fseek(f,offset,SEEK_SET);
			for(y=0;y<height;++y)						//(Notice 4bytes/col for some reason)
				for(x=0;x<width;++x)
				{
					BYTE byte;			
					fread(&byte,1,1,f);						//just read byte					
					for(int c=0;c<3;++c)
						Pixel(x,y,c)=cols[byte*4+2-c];	//and look up in the table
				}
			break;

		case(4):
			fread(cols,16*4,1,f);
			fseek(f,offset,SEEK_SET);
			for(y=0;y<256;++y)
				for(x=0;x<256;x+=2)
				{
					BYTE byte;
					fread(&byte,1,1,f);						//as above, but need to exract two
					for(c=0;c<3;++c)						//pixels from each byte
						Pixel(x,y,c)=cols[byte/16*4+2-c];
					for(c=0;c<3;++c)
						Pixel(x+1,y,c)=cols[byte%16*4+2-c];
				}
			break;

		case(1):
			fread(cols,8,1,f);
			fseek(f,offset,SEEK_SET);
			for(y=0;y<height;++y)
				for(x=0;x<width;x+=8)
				{
					BYTE byte;
					fread(&byte,1,1,f);
					//Every byte is eight pixels
					//so I'm shifting the byte to the relevant position, then masking out
					//all but the lowest bit in order to get the index into the colourtable.
					for(int x2=0;x2<8;++x2)
						for(int c=0;c<3;++c)
							Pixel(x+x2,y,c)=cols[((byte>>(7-x2))&1)*4+2-c];
				}
			break;

		default:
			fclose(f);
			delete[] tmpBytes;
			return BMPUNKNOWNFORMAT;
		}
		delete[] tmpBytes;

		if(ferror(f))
		{
			fclose(f);
			return BMPFILEERROR;
		}
		
		fclose(f);

		return BMPNOERROR;
	}

	//BMPError BMPLoad(std::string fname){BMPLoader bmp;return BMPLoad(fname,bmp);}

	#ifdef __gl_h
	BMPError BMPLoadGL(std::string fname)
	{
		BMPLoader bmp;
		BMPError e=BMPLoad(fname,bmp);
		if(e!=BMPNOERROR) return e;
			
		glEnable(GL_TEXTURE_2D);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,3,bmp.width,bmp.height,0,GL_RGB,GL_UNSIGNED_BYTE,bmp.bytes);

		return BMPNOERROR;
	}
	#endif
}