/*BMPLoader - loads Microsoft .bmp format
  Takes a filename, returns an array of RGB pixel data
  Loads:
  24bit bitmaps
  256 colour bitmaps
  16 colour bitmaps
  2 colour bitmaps  (Thanks to Charles Rabier)

  This code is designed for use in openGL programs, so bitmaps not correctly padded will not
  load properly, I believe this only applies to: 
  256cols if width is not a multiple of 4
  16cols if width is not a multiple of 8
  2cols if width is not a multiple of 32

  Sample code:

	BMPClass bmp;
	BMPLoad(fname,bmp);
	glTexImage2D(GL_TEXTURE_2D,0,3,bmp.width,bmp.height,0,GL_RGB,GL_UNSIGNED_BYTE,bmp.bytes);
*/

#pragma once


#include <string>

typedef unsigned char BYTE;
#define BMPError char
#define BMPNOTABITMAP 'b'	//Possible error flags
#define BMPNOOPEN 'o'
#define BMPFILEERROR 'f'
#define BMPBADINT 'i'
#define BMPNOERROR '\0'
#define BMPUNKNOWNFORMAT 'u'

using namespace std;

namespace SceneManagement
{
	class BMPLoader
	{
	public:
		int width,height;
		BYTE* bytes;											//OpenGL formatted pixels

		BMPLoader(void);
		~BMPLoader(void);

		BYTE& Pixel(int x,int y,int c);
		void AllocateMem();
		BMPError BMPLoad(std::string fname);					//Loads the bmp in fname, and puts the data in bmp
		std::string TranslateBMPError(BMPError err);			//Translates my error codes into English
		BMPError BMPLoadGL(std::string fname);					//Load and select in OpenGL
		void BMPaaa(std::string fname);
	};
}
