#include "stdafx.h"
#include "shader.h"
#include "shaderProgram.h"
#include "SceneManager.h"
#include "SceneOrigin.h"
#include "Sphere.h"
#include "Cube.h"
#include "Blended.h"
#include "dalkari2.h"
#include "FreeImage.h"
#include "imageManager.h"
#include "texgen.h"
#include "planet.h"
//#include "entity_Bump.h"
#include "my_entity_Bump.h"
#include "Space.h"
#include "light.h"
#include "sun.h"

namespace SceneManagement
{
	SceneManager SceneManager::SINGLETON_INSTANCE;

	SceneManager::SceneManager() 
	{
		camera = new Camera();
		mouse = new Mouse(camera);
		sceneSetting = new SceneSetting();

		Mouse::CALLBACK_INSTANCE = mouse;
		lastSelectedShape = -1;
		mode = 1;
	}

	SceneManager::~SceneManager() 
	{
		delete mouse;
		delete camera;
		delete sceneSetting;
		
		for (int i=0; i<sceneEntities.size(); i++)
			delete sceneEntities[i];

		for (int i=0; i<infoEntities.size(); i++)
			delete infoEntities[i];

		openGLIdleEntities.clear();
	}

	SceneManager* SceneManager::GetInstance() 
	{
		return &SINGLETON_INSTANCE;
	}

	void SceneManager::LoadTextures()
	{
		FreeImage_Initialise();

		// Rock texture
		FIBITMAP * image = ImageManager::GenericLoader("../textures/skala.bmp", 0);
		//FIBITMAP * image = ImageManager::GenericLoader("../textures/Geothermal Flats.jpg", 0);
		glGenTextures(1, &texs[2]);
		glBindTexture(GL_TEXTURE_2D, texs[2]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Grass texture
		image = ImageManager::GenericLoader("../textures/trava.bmp", 0);
		glGenTextures(1, &texs[0]);
		glBindTexture(GL_TEXTURE_2D, texs[0]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Road texture
		image = ImageManager::GenericLoader("../textures/cesty.bmp", 0);
		glGenTextures(1, &texs[1]);
		glBindTexture(GL_TEXTURE_2D, texs[1]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_LUMINANCE,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_LUMINANCE,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// 1D texture
		image = ImageManager::GenericLoader("../textures/1Dtexture.bmp", 0);
		glGenTextures(1, &tex_1d);
		glBindTexture(GL_TEXTURE_1D, tex_1d);
		glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage1D(GL_TEXTURE_1D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Jupiter texture
		image = ImageManager::GenericLoader("../textures/jupitermap.jpg", 0);
		glGenTextures(1, &texjupiter);
		glBindTexture(GL_TEXTURE_2D, texjupiter);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Sun texture
		image = ImageManager::GenericLoader("../textures/sunmap.jpg", 0);
		glGenTextures(1, &tSun);
		glBindTexture(GL_TEXTURE_2D, tSun);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Neptune texture
		image = ImageManager::GenericLoader("../textures/neptunemap.jpg", 0);
		glGenTextures(1, &texneptune);
		glBindTexture(GL_TEXTURE_2D, texneptune);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		image = ImageManager::GenericLoader("../textures/stonewall.bmp", 0);
                glGenTextures(1, &tDiffuse);
                glBindTexture(GL_TEXTURE_2D, tDiffuse);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, FreeImage_GetWidth(image), FreeImage_GetHeight(image), 0, GL_BGR, GL_UNSIGNED_BYTE, FreeImage_GetBits(image));
                FreeImage_Unload(image);

		image = ImageManager::GenericLoader("../textures/stonewallnormalmap.bmp", 0);
//		image = ImageManager::GenericLoader("../textures/stonewallnormalmap-gimp.bmp", 0);
//		image = ImageManager::GenericLoader("../textures/stonewallnormalmap-gimp.jpg", 0);
//		image = ImageManager::GenericLoader("../textures/skala-normal.jpg", 0);
                glGenTextures(1, &tNormal);
                glBindTexture(GL_TEXTURE_2D, tNormal);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, FreeImage_GetWidth(image), FreeImage_GetHeight(image), 0,
		GL_BGR,
//		GL_RGB,
		GL_UNSIGNED_BYTE, FreeImage_GetBits(image));
                FreeImage_Unload(image);

		image = ImageManager::GenericLoader("../textures/normalmap.bmp", 0);
                glGenTextures(1, &tNormal2);
                glBindTexture(GL_TEXTURE_2D, tNormal2);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, FreeImage_GetWidth(image), FreeImage_GetHeight(image), 0, GL_BGR, GL_UNSIGNED_BYTE, FreeImage_GetBits(image));
                FreeImage_Unload(image);

		// Saturn texture
		image = ImageManager::GenericLoader("../textures/saturnmap.jpg", 0);
		glGenTextures(1, &tSaturn[0]);
		glBindTexture(GL_TEXTURE_2D, tSaturn[0]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Saturn ring texture
		image = ImageManager::GenericLoader("../textures/saturnringcolor.jpg", 0);
		glGenTextures(1, &tSaturn[1]);
		glBindTexture(GL_TEXTURE_2D, tSaturn[1]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

		gluBuild2DMipmaps(GL_TEXTURE_2D,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Saturn ring transparency texture
		image = ImageManager::GenericLoader("../textures/saturnringpattern.gif", 0);
		glGenTextures(1, &tSaturn[2]);
		glBindTexture(GL_TEXTURE_2D, tSaturn[2]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

		gluBuild2DMipmaps(GL_TEXTURE_2D,
		GL_ALPHA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		GL_ALPHA,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Uranus texture
		image = ImageManager::GenericLoader("../textures/uranusmap.jpg", 0);
		glGenTextures(1, &tUranus[0]);
		glBindTexture(GL_TEXTURE_2D, tUranus[0]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Uranus ring texture
		image = ImageManager::GenericLoader("../textures/uranusringcolour.jpg", 0);
		glGenTextures(1, &tUranus[1]);
		glBindTexture(GL_TEXTURE_2D, tUranus[1]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

		gluBuild2DMipmaps(GL_TEXTURE_2D,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Uranus ring transparency texture
		image = ImageManager::GenericLoader("../textures/uranusringtrans.gif", 0);
		glGenTextures(1, &tUranus[2]);
		glBindTexture(GL_TEXTURE_2D, tUranus[2]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

		gluBuild2DMipmaps(GL_TEXTURE_2D,
		GL_ALPHA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		GL_ALPHA,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);



		// Space texture
		image = ImageManager::GenericLoader("../textures/space4.jpg", 0);
		glGenTextures(1, &tSpace);
		glBindTexture(GL_TEXTURE_2D, tSpace);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		gluBuild2DMipmaps(GL_TEXTURE_2D,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));

		image = ImageManager::GenericLoader("../textures/stars960e.jpg", 0);
		glGenTextures(1, &tStars);
		glBindTexture(GL_TEXTURE_2D, tStars);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		gluBuild2DMipmaps(GL_TEXTURE_2D,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
/*		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
*/
		FreeImage_Unload(image);

		// Mercury texture
		image = ImageManager::GenericLoader("../textures/mercurymap.jpg", 0);
		glGenTextures(1, &tMercury);
		glBindTexture(GL_TEXTURE_2D, tMercury);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Venus texture
		image = ImageManager::GenericLoader("../textures/venusmap.jpg", 0);
		glGenTextures(1, &tVenus);
		glBindTexture(GL_TEXTURE_2D, tVenus);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Earth texture
		image = ImageManager::GenericLoader("../textures/earthmap1k.jpg", 0);
		glGenTextures(1, &tEarth);
		glBindTexture(GL_TEXTURE_2D, tEarth);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Earth Normal texture
		image = ImageManager::GenericLoader("../textures/Earth_NormalMap_2.jpg", 0);
		glGenTextures(1, &tEarthNormal);
		glBindTexture(GL_TEXTURE_2D, tEarthNormal);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);


		// Mars texture
		image = ImageManager::GenericLoader("../textures/mars_1k_color.jpg", 0);
		glGenTextures(1, &tMars);
		glBindTexture(GL_TEXTURE_2D, tMars);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);

		// Mars normal texture
		image = ImageManager::GenericLoader("../textures/mars_1k_normal.jpg", 0);
		glGenTextures(1, &tMarsNormal);
		glBindTexture(GL_TEXTURE_2D, tMarsNormal);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);


		// Jupiter texture
		image = ImageManager::GenericLoader("../textures/jupitermap.jpg", 0);
		glGenTextures(1, &tJupiter);
		glBindTexture(GL_TEXTURE_2D, tJupiter);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		FreeImage_GetWidth(image),
		FreeImage_GetHeight(image),
		0,
		GL_BGR,
		GL_UNSIGNED_BYTE,
		FreeImage_GetBits(image));
		FreeImage_Unload(image);


		FreeImage_DeInitialise();
	}

//-----------------------------------------------------------------------------------------------------------------
//OpenGL Initialization
//-----------------------------------------------------------------------------------------------------------------
	void SceneManager::Init(int argc, char* argv[]) 
	{
		glutInit(&argc, argv);

		glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
		glutInitWindowSize(800, 600);
		glutInitWindowPosition(0, 0);

		mainWindow = glutCreateWindow("Computer graphics II - 3D engine");

		glutFullScreen();

		// Display and Idle
		glutDisplayFunc(SceneManager::DisplayCB);
		glutReshapeFunc(SceneManager::ReshapeCB);
		glutIdleFunc(SceneManager::IdleCB);

		// Keyboard
		glutKeyboardFunc(SceneManager::KeyTypedCB);
		glutSpecialFunc(SceneManager::InputKeyCB);

		// Mouse stuff
		glutMouseFunc(Mouse::MouseClickCB);
		glutMotionFunc(Mouse::ActiveMouseMoveCB);
		glutPassiveMotionFunc(Mouse::PassiveMouseMoveCB);

		//GLfloat ambient[] = {0.1f, 0.2f, 0.1f, 1};
		//GLfloat diffuse[] = {1, 1, 1, 1};
		//GLfloat specular[] = {0.1f, 0.1f, 0.1f, 1};
		//GLfloat shininess = 1;

		//glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
		//glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
		//glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
		//glMaterialf(GL_FRONT, GL_SHININESS, shininess);

		glewInit();
		LoadTextures();

		glEnable(GL_DEPTH_TEST);
		glShadeModel(GL_SMOOTH);
//		glEnable(GL_CULL_FACE);
//		glCullFace(GL_BACK);
#if 0
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
		glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
		glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
#endif

		//Inicializace objektu ve scene
		// 1 AU =  150 000 000 km
		// Size -        1 : 4 000
		// Distance -   80 : 1
		// Sun     - 700 000 km, 0,0 AU
		// Mercury -   2 440 km, 0,4 AU
		// Venus   -   6 052 km, 0,7 AU
		// Earth   -   6 371 km, 1,0 AU
		// Mars    -   3 390 km, 1,5 AU
		// Jupiter -  69 911 km, 5,2 AU
		// Saturn  -  58 232 km, 9,5 AU
		// Uranus  -  25 362 km, 19,6 AU
		GLfloat sunPos[3] = {0, 0, 0}, sunSize = 17.5;
		GLfloat mercuryPos[3] = {32, 0, 0}, mercurySize = 0.61;
		GLfloat venusPos[3] = {56, 0, 0}, venusSize = 1.513;
		GLfloat earthPos[3] = {80, 0, 0}, earthSize = 1.59275;
		GLfloat marsPos[3] = {120, 0, 0}, marsSize = 0.8475;
		GLfloat jupiterPos[3] = {416, 0, 0}, jupiterSize = 17.47775;
		GLfloat saturnPos[3] = {760, 0, 0}, saturnSize = 14.558, saturnRot = 30;
		GLfloat uranusPos[3] = {1584, 0, 0}, uranusSize = 6.3405, uranusRot = 76;


//		infoEntities.push_back(new SceneOrigin());
//		sceneEntities.push_back(new Space(tStars));
//		sceneEntities.push_back(new sun(tSun, 15, sunPos));
//		sceneEntities.push_back(new Light());
//		sceneEntities.push_back(new Space(tSpace));
//		sceneEntities.push_back(new Sphere());
//		sceneEntities.push_back(new Blended(texs[2]));
//		sceneEntities.push_back(new texgen(tex_1d));
//		sceneEntities.push_back(new planet(texjupiter));
//		sceneEntities.push_back(new planet(texneptune));
//		sceneEntities.push_back(new planet(tMercury, mercurySize, mercuryPos, 0, "Mercury"));
//		sceneEntities.push_back(new planet(tVenus, venusSize, venusPos, 0, "Venus"));
//		sceneEntities.push_back(new planet(tEarth, earthSize, earthPos, 7.155, "Earth"));
//		sceneEntities.push_back(new planet(tMars, marsSize, marsPos, 0, "Mars"));
//		sceneEntities.push_back(new planet(tJupiter, jupiterSize, jupiterPos, 0, "Jupiter"));
//		sceneEntities.push_back(new planet(tSaturn, saturnSize, saturnPos, saturnRot, "Saturn"));
//		sceneEntities.push_back(new planet(tUranus, uranusSize, uranusPos, uranusRot, "Uranus"));
//		sceneEntities.push_back(new Entity_Bump(tDiffuse, tNormal));
//		sceneEntities.push_back(new My_Entity_Bump(tDiffuse, tNormal));
		sceneEntities.push_back(new My_Entity_Bump(tEarth, tEarthNormal));
//		sceneEntities.push_back(new Entity_Bump(texneptune, tNormal2));
//		sceneEntities.push_back(new Dalkari2());
//		sceneEntities.push_back(new Blended(tDiffuse));
		
/*		Cube *c = new Cube(1,1,1);
		sceneEntities.push_back(c);
		openGLIdleEntities.push_back(c);
*/

		//Experimentalni load shaderu
		//vertex_shader = new Shader(VERTEX_SHADER);
		//vertex_shader->openFromFile("D:\\plastic.vert");
		//fragment_shader = new Shader(FRAGMENT_SHADER);
		//fragment_shader->openFromFile("D:\\plastic.frag");

		//shader = new ShaderProgram(vertex_shader, fragment_shader);
		//shader->enable();
	}


//-----------------------------------------------------------------------------------------------------------------
//OpenGL Display and Idle functions
//-----------------------------------------------------------------------------------------------------------------

	void SceneManager::Display() 
	{
		counter.markFrameStart();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		camera->CameraZoomAndRotate();
		camera->CameraPan();

		for (int i=0; i<infoEntities.size(); i++)
			infoEntities[i]->Draw();

		for (int i=0; i<sceneEntities.size(); i++)
			sceneEntities[i]->Draw();

		glFlush();
		glutSwapBuffers();
		counter.markFrameEnd();
	}

	void SceneManager::DisplayCB() 
	{
		SceneManager::GetInstance()->Display();
	}
	
	void SceneManager::Redisplay() 
	{
		glutSetWindow(mainWindow);
		glutPostRedisplay();
	}

	void SceneManager::Idle() 
	{
		if (openGLIdleEntities.size() > 0)
		{
			for (int i=0; i<openGLIdleEntities.size(); i++)		//Volaji se vsechny ukazatele na idle funkce.
			{
				openGLIdleEntities[i]->Idle();						
			}
		}
		glutPostRedisplay();
	}

	void SceneManager::IdleCB() 
	{
		SceneManager::GetInstance()->Idle();					
	}

//-----------------------------------------------------------------------------------------------------------------
//OpenGL Reshape functions
//-----------------------------------------------------------------------------------------------------------------

	void SceneManager::Reshape(int width, int height) 
	{
		float aspectRatio = 1.33f;
		height            = height <= 0 ? 1 : height;
		aspectRatio       = (float)width/height;

		glViewport(0, 0, width, height);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
//		gluPerspective(90.0f, aspectRatio, .1f, 1500.0f);
		gluPerspective(65.0f, aspectRatio, .1f, 6000.0f);
		Display();
	}

	void SceneManager::ReshapeCB(int width, int height) 
	{
		SceneManager::GetInstance()->Reshape(width, height);
	}

//-----------------------------------------------------------------------------------------------------------------
//OpenGL Picking
//-----------------------------------------------------------------------------------------------------------------

	void SceneManager::Pick(int &x, int &y) 
	{
		
	}

	void SceneManager::PickCB(int &x, int &y) 
	{
		SceneManager::GetInstance()->Pick(x,y);
	}

//-----------------------------------------------------------------------------------------------------------------
//OpenGL Input functions
//-----------------------------------------------------------------------------------------------------------------

	void SceneManager::KeyTyped(unsigned char key, int x, int y)
	{
	  if (key == 27)
		exit(0);
	  if (key == 'w')
	  {
            switch (mode)
		{
		case 1: glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); mode++; break;
		case 2: glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); mode++; break;
		case 3: glPolygonMode(GL_FRONT_AND_BACK, GL_POINT); mode=1; break;
		}
	  }
	  if (key == ' ')
	  {
		delete camera; camera = new Camera();
	  }
	}

	void SceneManager::KeyTypedCB(unsigned char key, int x, int y)
	{
		SceneManager::GetInstance()->KeyTyped(key,x,y);
	}

	void SceneManager::InputKey(int key, int x, int y)
	{
	}

	void SceneManager::InputKeyCB(int key, int x, int y)
	{
		SceneManager::GetInstance()->InputKey(key,x,y);
	}
}
