#pragma once
#include <vector>
#include "TimeUtils.h"
#include "Camera.h"
#include "Mouse.h"
#include "SceneSetting.h"
#include "Entity.h"
#include "shaderProgram.h"

class Entity;

using namespace std;

namespace SceneManagement
{
	class Mouse;
	class SceneManager
	{
	public:		//Global State variables
		int		lastSelectedShape;
		float	mouse_z;
		vector<Entity*>	sceneEntities;
		vector<Entity*>	infoEntities;
		ShaderProgram *shader;
		Shader *vertex_shader;
		Shader *fragment_shader;


	private:
		static		SceneManager SINGLETON_INSTANCE;
		
		GLdouble	mvMatrix[16];		//Modelovaci matice
		GLdouble	projMatrix[16];		//Projekcni matice
		GLint		viewport[4];		//Viewport - x,y,width, height

		GLdouble pointerX, pointerY, pointerZ;

		vector<Entity*> openGLIdleEntities;

		SceneSetting *sceneSetting;
		FPSCounter    counter;
		int mode;

	public:
		GLuint		mainWindow;
		Camera*		camera;
		Mouse*		mouse;

		SceneManager();
		~SceneManager();
		//textures
		GLuint texs[3];
		GLuint tex_1d;
		GLuint texjupiter;
		GLuint texneptune;
		GLuint tDiffuse;
		GLuint tNormal;
		GLuint tNormal2;
		GLuint tSaturn[3];
		GLuint tUranus[3];
		GLuint tMercury;
		GLuint tVenus;
		GLuint tEarth;
		GLuint tEarthNormal;
		GLuint tMars;
		GLuint tMarsNormal;
		GLuint tJupiter;
		GLuint tSun;
		GLuint tSpace;
		GLuint tStars;

		static		SceneManager* GetInstance();

		static void	IdleCB();
		static void	DisplayCB();
		static void	ReshapeCB(int width, int height);
		static void PickCB(int &x, int &y);
		static void	KeyTypedCB(unsigned char key, int x, int y);
		static void	InputKeyCB(int key, int x, int y);

		void		LoadTextures();
		void		Init(int argc, char* argv[]);
		void		Display();
		void		Reshape(int width, int height);
		void		Pick(int &x, int &y);
		void		KeyTyped(unsigned char key, int x, int y);
		void		InputKey(int key, int x, int y);

		void		Redisplay();
		void		Idle();
	};
}
