#include "stdafx.h"
#include "planet.h"
#include "glut.h"
#include "glu.h"

namespace SceneManagement
{
	planet::planet(GLuint texture, GLfloat psize, GLfloat pposition[3], GLfloat rot, char* pname)
	{
		qPlanet = gluNewQuadric();
		haveRing = false;
		tex = texture;
		cell = new Cell();
		size = psize;
		position[0] = pposition[0];
		position[1] = pposition[1];
		position[2] = pposition[2];
		strcpy(name, pname);
		rotation = rot;
	}

	planet::planet(GLuint* texs, GLfloat psize, GLfloat pposition[3], GLfloat rot, char* pname)
	{
		qPlanet = gluNewQuadric();
		haveRing = true; createRing();
		tex = texs[0];
		texRing = texs[1];
		texRingTrans = texs[2];
		cell = new Cell();
		size = psize;
		position[0] = pposition[0];
		position[1] = pposition[1];
		position[2] = pposition[2];
		strcpy(name, pname);
		rotation = rot;
	}

	void planet::createRing()
	{
		int a = 0; int b = 0;
		do
		{ //Hopefully OK
			ringVertices[a] = cos(2*b*M_PI/RING_SLICES)*RING_MIN;
			ringTexCoords[a] = 1;
			a++;

			ringVertices[a] = sin(2*b*M_PI/RING_SLICES)*RING_MIN;
			ringTexCoords[a] = b/(float)RING_SLICES;
			a++;

			ringVertices[a] = cos(2*b*M_PI/RING_SLICES)*RING_MAX;
			ringTexCoords[a] = 0;
			a++;


			ringVertices[a] = sin(2*b*M_PI/RING_SLICES)*RING_MAX;
			ringTexCoords[a] = (b)/(float)RING_SLICES;
			a++; b++;
		} while (a < (RING_SLICES +1) * 4);
		for (a=0; a < (RING_SLICES+1)*6; a+=3)
		{
			ringNormals[a] = 0;
			ringNormals[a+1] = 0;
			ringNormals[a+2] = 1;
		}
	}

	planet::~planet()
	{
		gluDeleteQuadric(qPlanet);
		delete cell;
	}

	void planet::Draw()
	{
	glPushAttrib(GL_COLOR_MATERIAL | GL_LIGHTING);
	
	// For lighting, pull our material colors from the vertex color...
		float mAmbientDiffuse[4] = { 0.0, 0.0, 0.0, 1 };
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mAmbientDiffuse);

		glEnable (GL_BLEND); glEnable(GL_DEPTH_TEST);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		static float angle = 0.0; angle += 0.025;
		glPushMatrix();
//		glRotatef(angle/5, 0, 0, 1);  //otaceni planety kolem slunce
		glTranslatef(position[0], position[1], position[2]);
		glPushMatrix();

		glRotatef(rotation, 0, -1, 0);  //natoceni planety
		glRotatef(angle, 0, 0, 1);  //otaceni planety kolem vlastni osy
		glScalef(size, size, size);  //velikost planety
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mAmbientDiffuse);
		glColorMaterial( GL_FRONT, GL_DIFFUSE );
		glEnable( GL_COLOR_MATERIAL );
		glColor4f(1.0, 1.0, 1.0, 1.0);
		gluQuadricTexture(qPlanet, true);
		gluSphere(qPlanet, 1.0, RING_SLICES/2, RING_SLICES/2);
		glDisable(GL_TEXTURE_2D);
		if(haveRing)
		{
			glDisable(GL_CULL_FACE);
			glClientActiveTexture(GL_TEXTURE0);
			glActiveTexture(GL_TEXTURE0);
		        glEnable(GL_TEXTURE_2D);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glTexCoordPointer(2, GL_FLOAT, 0, ringTexCoords);
			glBindTexture(GL_TEXTURE_2D, texRing);
			glClientActiveTexture(GL_TEXTURE1);
			glActiveTexture(GL_TEXTURE1);
		        glEnable(GL_TEXTURE_2D);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glTexCoordPointer(2, GL_FLOAT, 0, ringTexCoords);
			glBindTexture(GL_TEXTURE_2D, texRingTrans);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
			glEnableClientState(GL_VERTEX_ARRAY);
			glEnableClientState(GL_NORMAL_ARRAY);
			glVertexPointer(2, GL_FLOAT, 0, ringVertices);
			glNormalPointer(GL_FLOAT, 0, ringNormals);
			glDrawArrays(GL_QUAD_STRIP, 0, (RING_SLICES + 1) * 2);
			glDisableClientState(GL_VERTEX_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glActiveTexture(GL_TEXTURE1);
			glClientActiveTexture(GL_TEXTURE1);
			glDisable(GL_TEXTURE_2D);
			glActiveTexture(GL_TEXTURE0);
			glClientActiveTexture(GL_TEXTURE0);
			glDisable(GL_TEXTURE_2D);
			glEnable(GL_CULL_FACE);
		}
		glDisable (GL_BLEND);
		glPopMatrix();

		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);
		glColor4f(1.0, 1.0, 1.0, 1.0);
		cell->DrawString3D(0, 0, 1.333*size, name);
		glEnable(GL_LIGHTING);
		glEnable(GL_DEPTH_TEST);


		glPopMatrix();
		glPopAttrib();
	}
}
