#include "shader.h"

#include <iostream>
#include <fstream>
#include <string>

using namespace std;
using namespace SceneManagement;

Shader::Shader(ShaderType type)
{
	shaderType = type;
	shaderState = SceneManagement::UNKNOWN;
	shaderObject=0;
}

Shader::~Shader()
{
  glDeleteShader(shaderObject);					//glDeleteObjectARB(shaderObject);
}

void Shader::create()
{
	if (shaderState == SceneManagement::CREATED) return;
  
	shaderObject = glCreateShader(shaderType);   //glCreateShaderObjectARB(shaderType);
	shaderState = SceneManagement::CREATED;
}

char *Shader::getInfoLog()
{
	char *infoLog;
	int infoLogLength;

	//glGetObjectParameterivARB(object,GL_OBJECT_INFO_LOG_LENGTH_ARB,(GLint *)&infoLogLength);
	glGetShaderiv(shaderObject,	GL_INFO_LOG_LENGTH, &infoLogLength);

	infoLog=new char[infoLogLength+1];
	//glGetInfoLogARB(object,infoLogLength,(GLsizei *)&infoLogLength,infoLog);
	glGetShaderInfoLog(shaderObject, infoLogLength, &infoLogLength, infoLog);
	return infoLog;
}

void Shader::openFromFile(const char *filename)
{
	ifstream input(filename);

	if (!input.is_open())
		throw "Shader::exception(47):  unable to load shader file!";

	if (shaderObject==0) 
		create();

	/*  No vertex or fragment program should ever be longer than 512 lines by 255 characters. */
	const int MAX_LINES=512;
	const int MAX_LINE_LENGTH=256;   // 255 + NULL terminator
	char *program = (char*)malloc(MAX_LINES*MAX_LINE_LENGTH);

	const GLcharARB **programPtr=(const GLcharARB **)&program;
	int len;
	char *ptr=program;

	while (!input.eof())
	{
		input.getline(ptr,MAX_LINE_LENGTH);
		len=strlen(ptr);
		ptr[len]='\n';
		ptr+=len+1;
	}
	ptr[0]=0;

	cout << program << endl;

	input.close();

	glShaderSourceARB(shaderObject,1,programPtr,NULL);
	shaderState = SceneManagement::LOADED;

	glCompileShaderARB(shaderObject);

	int status = 0;
	//glGetObjectParameterivARB(shaderObject,GL_OBJECT_COMPILE_STATUS_ARB,(GLint *)&status);
	glGetShaderiv(shaderObject, GL_COMPILE_STATUS, (GLint *)&status);
	if (!status)
	throw "Shader::exception(80):  shader did not compile!";

	shaderState = SceneManagement::COMPILED;

	free(program);
}


void Shader::openFromArray(const char *data)
{
	if (data)
		create();

	const GLcharARB **programPtr=(const GLcharARB **)&data;
	glShaderSourceARB(shaderObject,1,programPtr,NULL);
	shaderState = SceneManagement::LOADED;

	glCompileShaderARB(shaderObject);

	int status = 0;
	//glGetObjectParameterivARB(shaderObject,GL_OBJECT_COMPILE_STATUS_ARB,(GLint *)&status);
	glGetShaderiv(shaderObject, GL_COMPILE_STATUS, (GLint *)&status);
	if (!status)
	throw "Shader::exception(80):  shader did not compile!";

	shaderState = SceneManagement::COMPILED;
}
