#include "shaderProgram.h"
#include <iostream>

using namespace std;
using namespace SceneManagement;

ShaderProgram::ShaderProgram(Shader *vertexShader, Shader *fragmentShader)
{
	//Vytvoreni programu
	programObject= glCreateProgram();   //glCreateProgramObjectARB();

	if (vertexShader)
	{
		this->vertexShader = vertexShader;
		enableVertexShader();
	}
	if (fragmentShader)
	{
		this->fragmentShader = fragmentShader;
		enableFragmentShader();
	}
	enabled=false;
}

ShaderProgram::~ShaderProgram()
{
  disableShaders();
  glDeleteProgram(programObject);  //glDeleteObjectARB(programObject);
  delete vertexShader;
  delete fragmentShader;
}

char *ShaderProgram::getInfoLog()
{
  char *infoLog;
  int infoLogLength;
  
  //glGetObjectParameterivARB(object,GL_OBJECT_INFO_LOG_LENGTH_ARB,(GLint *)&infoLogLength);
  glGetProgramiv(programObject, GL_INFO_LOG_LENGTH, &infoLogLength);

  infoLog=new char[infoLogLength+1];
  //glGetInfoLogARB(object,infoLogLength,(GLsizei *)&infoLogLength,infoLog);
	glGetProgramInfoLog(programObject, infoLogLength, &infoLogLength, infoLog);
  return infoLog;
}

void ShaderProgram::enable()
{
  disable();
  //glUseProgramObjectARB(programObject);
  glUseProgram(programObject);
  enabled=true;
}

void ShaderProgram::enableShaders()
{
  enableVertexShader();
  enableFragmentShader();
}

void ShaderProgram::enableVertexShader()
{
	disableVertexShader();
	glAttachShader(programObject,vertexShader->shaderObject);				//glAttachObjectARB(programObject,vertexShader.shaderObject);
	glLinkProgramARB(programObject);
	vertexShaderUsed=true;

	int status = 0;
	glGetProgramiv(programObject, GL_LINK_STATUS, (GLint *)&status);
	if (!status)
	throw "Shader::exception(80):  shader did not compile!";
}

void ShaderProgram::enableFragmentShader()
{
	disableFragmentShader();

	glAttachShader(programObject,fragmentShader->shaderObject);			//glAttachObjectARB(programObject,fragmentShader.shaderObject);
	glLinkProgramARB(programObject);
	fragmentShaderUsed=true;

	int status = 0;
	glGetProgramiv(programObject, GL_LINK_STATUS, (GLint *)&status);
	if (!status)
	throw "Shader::exception(80):  shader did not compile!";
}

void ShaderProgram::disable()
{
  //glUseProgramObjectARB(0); 
  glUseProgram(0); 
  if (enabled) enabled=false;
}

void ShaderProgram::disableShaders()
{
  disableVertexShader();
  disableFragmentShader();
}

void ShaderProgram::disableVertexShader()
{
  if (vertexShaderUsed)
  {
	glDetachShader(programObject,vertexShader->shaderObject);		//glDetachObjectARB(programObject,vertexShader.shaderObject);
    vertexShaderUsed=false;
  }
}

void ShaderProgram::disableFragmentShader()
{
  if (fragmentShaderUsed)
  {
    glDetachShader(programObject,fragmentShader->shaderObject);		// glDetachObjectARB(programObject,fragmentShader.shaderObject);
    fragmentShaderUsed=false;
  }
}
